#ifndef _THIRD_PARTY_COMPONENT_H_
#define _THIRD_PARTY_COMPONENT_H_

#include <string>

namespace log
{
    void trace(const char* msg);
}

namespace gui
{

void initLibrary();
void uninitLibrary();

enum RET_CODE 
{
    RC_CANCEL,
    RC_OK
};

class Dialog
{
public:
    virtual ~Dialog() {}
    RET_CODE doModal();

protected:
    virtual RET_CODE doContent() = 0;

private:
    void scrollAway();
};

struct GreetingDlg : public Dialog
{
    RET_CODE doContent();
};

struct LoginScreenDlg : public Dialog
{
    RET_CODE doContent();

    std::string _pwd;
};

struct BadPasswordDlg : public Dialog
{
    RET_CODE doContent();
};

struct LockScreenDlg : public Dialog
{
    RET_CODE doContent();

    std::string _pwd;
};

}   // end of gui namespace

namespace kernel
{

class Guard
{
public:
    static Guard*   getInstance(const char* someInputArgument);
    static void     releaseInstance(Guard*); 

public:
    bool checkUserPwd(const char*) const;
    bool checkAdminPwd(const char*) const;

private:
    Guard();    // kernel library has restriction: a client is not able to create object of this type in stack.
                // why?
                // to make client's life more difficult ;-(
                // by these code we demonstrate third party components very often have bad design with relation to usability
};

}   // end of kernel namespace



#endif  // !_THIRD_PARTY_COMPONENT_H_