#include <iostream>
#include "../fsm.h"

using namespace std;

enum STATE
{
    Locked,
    Unlocked
};

template <typename T> T cast(int state);
template <> const char* cast<const char*>(int val)
{
    switch(val)
    {
        case Locked   :   return "Locked";
        case Unlocked :   return "Unlocked";
        default:            return "";
    }
}


enum EVENT
{
    COIN,
    PASS
};

struct Actions
{
    void lock()         { cout << "action--->lock()" << endl; }
    void unlock()       { cout << "action--->unlock()" << endl; }
    void alarm()        { cout << "action--->alarm()" << endl; }
    void thankyou()     { cout << "action--->thankyou()" << endl; }
};

typedef TYPELIST_4
(
        FSM_TRANSITION(Locked,   COIN, Unlocked, Actions,  unlock), 
        FSM_TRANSITION(Locked,   PASS, Locked,   Actions,  alarm), 
        FSM_TRANSITION(Unlocked, PASS, Locked,   Actions,  lock),
        FSM_TRANSITION(Unlocked, COIN, Unlocked, Actions,  thankyou)
) 
TransitionTable;


class TurnstileSM : public fsm::Builder<TransitionTable>
{
}; 

int main(int argc, char* argv[])
{
    TurnstileSM machine;
    cout << "SM current state: " << cast<const char*>(machine.state()) << endl << endl;

    cout << "Client sends event : COIN" << endl;
    machine.postEvent(COIN);
    cout << "SM current state: " << cast<const char*>(machine.state()) << endl << endl;

    cout << "Client sends event : PASS" << endl;
    machine.postEvent(PASS);
    cout << "SM current state: " << cast<const char*>(machine.state()) << endl << endl;

    cout << "Client sends event : PASS" << endl;
    machine.postEvent(PASS);
    cout << "SM current state: " << cast<const char*>(machine.state()) << endl << endl;

    cout << "Client sends event : COIN" << endl;
    machine.postEvent(COIN);
    cout << "SM current state: " << cast<const char*>(machine.state()) << endl << endl;

    cout << "Client sends event : COIN" << endl;
    machine.postEvent(COIN);
    cout << "SM current state: " << cast<const char*>(machine.state()) << endl << endl;

    cout << "Client sends unknown event: 10" << endl;
    // if you define your SM as fsm::Builder<TransitionTable,STATE,EVENT>, next line will not be compilable
    // error: cannot canvert int to EVENT type
    machine.postEvent(10);
    cout << "SM current state: " << cast<const char*>(machine.state()) << endl << endl;

    cout << "press any key and <enter> to exit" << endl;
    char ch; cin >> ch;

	return 0;
}


