#include <iostream>
#include "../fsm.h"

using namespace std;

enum STATE
{
    S_LOCKED,
    S_UNLOCKED
};

template <typename T> T cast(STATE state);
template <> const char* cast<const char*>(STATE val)
{
    switch(val)
    {
        case S_LOCKED   :   return "S_LOCKED";
        case S_UNLOCKED :   return "S_UNLOCKED";
        default:            return "UNKNOWN STATE";
    }
}

enum EVENT
{
    E_COIN,
    E_PASS,
    E_UNKNOWN
};

template <typename T> T cast(EVENT state);
template <> const char* cast<const char*>(EVENT val)
{
    switch(val)
    {
        case E_COIN :   return "E_COIN";
        case E_PASS :   return "E_PASS";
        default:        return "UNKNOWN EVENT";
    }
}

struct Actions
{
    void lock()         { cout << "action--->lock()" << endl; }
    void unlock()       { cout << "action--->unlock()" << endl; }
    void alarm()        { cout << "action--->alarm()" << endl; }
    void thankyou()     { cout << "action--->thankyou()" << endl; }
};

typedef TYPELIST_4(
        FSM_TRANSITION           (S_LOCKED,   E_COIN, S_UNLOCKED, Actions,  unlock), 
        FSM_TRANSITION           (S_LOCKED,   E_PASS, S_LOCKED,   Actions,  alarm), 
        FSM_TRANSITION           (S_UNLOCKED, E_PASS, S_LOCKED,   Actions,  lock),
        FSM_TRANSITION_NO_ACTION (S_UNLOCKED, E_COIN, S_UNLOCKED)
) 
TransitionTable;

template
<
    typename STATE_T,
    typename EVENT_T
>
class Monitor
{
public:
    void on_received(STATE_T currentState, EVENT_T receivedEvent) 
            {
                cout << cast<const char*>(receivedEvent) 
                     << " event was received. Current state is " 
                     << cast<const char*>(currentState)
                     << endl;                     
            }
    void on_state_changed(STATE_T currentState, STATE_T previousState, EVENT_T overEvent )
            {
                if(currentState != previousState)
                    cout << "state was changed from " 
                        << cast<const char*>(previousState) 
                        << " to " 
                        << cast<const char*>(currentState)  
                        << endl;
                else
                    cout << "state remained unchangeable" << endl; 
            }
    void on_completed(STATE_T, STATE_T, EVENT_T )
            {
                    cout << "transition completed." << endl; 
            }
    void unhandled_event(STATE_T currentState, EVENT_T receivedEvent)
            {
                cout << "there is no transition accepted " 
                     << cast<const char*>(receivedEvent) 
                     << " event on " 
                     << cast<const char*>(currentState)
                     << " state." 
                     << endl;
            }
};

class TurnstileSM : public fsm::Builder<TransitionTable,STATE,EVENT,Monitor>
{
}; 


int main(int argc, char* argv[])
{
    TurnstileSM machine;
    cout << "SM current state: " << cast<const char*>(machine.state()) << endl << endl;

    machine.postEvent(E_COIN);
    cout << endl;
    machine.postEvent(E_PASS);
    cout << endl;
    machine.postEvent(E_PASS);
    cout << endl;
    machine.postEvent(E_COIN);
    cout << endl;
    machine.postEvent(E_COIN);
    cout << endl;

    machine.postEvent(E_UNKNOWN);
    cout << endl;

    cout << "press any key and <enter> to exit" << endl;
    char ch; cin >> ch;

	return 0;
}

